/*	$Id: extern.h,v 1.85 2015/06/06 21:00:42 kristaps Exp $ */
/*
 * Copyright (c) 2014, 2015 Kristaps Dzonsons <kristaps@kcons.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef EXTERN_H
#define EXTERN_H

/*
 * When rendering the skybox, this corresponds to the face of the skybox
 * around our cubic space.
 */
enum	face {
	FACE_POSX,
	FACE_POSY,
	FACE_POSZ,
	FACE_NEGX,
	FACE_NEGY,
	FACE_NEGZ,
	FACE__MAX
};

/*
 * A node on the HUD display.
 */
enum 	hudtype {
	HUD_AVGAGE = 0,
	HUD_ALIGN,
	HUD_ALIGNCLOCK,
	HUD_ALIGNFLOCK,
	HUD_CAMERA,
	HUD_COHESION,
	HUD_FLOCKSZ,
	HUD_FPS,
	HUD_FRAMES,
	HUD_LIGHT0,
	HUD_MATCHBOX,
	HUD_MATCHES,
	HUD_PLAY_INTERVAL,
	HUD_SCATTERING,
	HUD_SELFBOX,
	HUD_SEPARATION,
	HUD_SPEED_MAX,
	HUD_SPEED_MIN,
	HUD_TEND_RDEST,
	HUD_TEND_ROOST,
	HUD_WINDOW,
	HUD__MAX
};

/*
 * Where a HUD entry is drawn on the screen.
 * We use this when (e.g.) intercepting mouse-presses on the screen
 * while the HUD is drawn.
 */
struct	hudnode {
	/*
	 * Dimensions where a mouse-click will "edit" this.
	 * Not all entries are editable.
	 */
	GLint		 miny;
	GLint		 maxy;
	GLint		 minx;
	GLint		 maxx;
	/*
	 * If edit == 0, no editing.
	 * If edit == 1, we're mouse-overing.
	 * If edit == 2, we've clicked it.
	 */
	int		 edit;
	enum hudtype	 type;
};

#define	WING_SIZE	 25 /* Number of wing positions. */

enum	hudview {
	HUDVIEW_BIG = 1,
	HUDVIEW_MINI,
	HUDVIEW_LASTPLAY,
};

/*
 * This structure consists of basically everything.
 */
struct	args {
	struct timeval	 tv; /* last update (display mode) */
	size_t		 height; /* height of viewport */
	size_t		 width; /* width of viewport */
	size_t		 heightsv; 
	size_t		 widthsv;
	int		 fullscreen;
	GLfloat		*data; /* pixel data buffer */
	size_t		 maxframes; /* maximum frames before exiting */
	size_t		 maxmatches; /* maximum matches before exiting */
	size_t		 frame; /* current frame number (from 0) */
	size_t		 fps; /* frame-per-second (record mode) */
	double		 fps_real; /* actual fps (record mode) */
	double		 refresh; /* refresh rate (display mode) */
	double		 scale; /* bird scale (default 1.0) */
	char		*filebuf; /* buffer for saving filename */
	size_t		 filesz; /* size of file file buffer */
	int		 pause; /* whether paused (display mode) */
	enum hudview	 drawhud; /* whether we should HUD */
	int		 drawaxes; /* whether not to draw texels */
	int		 drawnotexs; /* whether not to draw textures */
	int		 drawnoobjs; /* whether not to draw objects */
	struct hudnode	 hudnodes[HUD__MAX]; /* used for HUD work */
	double		 cradius; /* camera radius */
	int		 cvradius; /* velocity of camera radius */
	double		 cx; /* camera x (horizontal) position */
	double		 cz; /* camera z (horizontal) position */
	double		 cy; /* camera y (vertical) position */
	double		 cxz; /* camera horizontal angle */
	double		 cxy; /* camera vertical angle */
	double		 cvxz; /* horizontal angular velocity */
	double		 cvxy; /* vertical angular velocity */
	int		 autotrack; /* auto-track centre? */
	double		 ground; /* ground position */
	GLfloat		 lightpos[4]; /* light position */
	GLuint		 bgid[6]; /* sky box textures */
	float		*birdVerts[WING_SIZE]; /* wing vertex */
	float		*birdNormals[WING_SIZE]; /* wing norms */
	size_t		 birdNumVerts[WING_SIZE]; /* faces */
	Mix_Chunk	*noise; /* if we're playing noises */
	int		 noisechan; /* channel for noises */
	struct game 	*g; /* the game itself */
	FILE		*ffmpeg; /* if we're recording */
};

/*
 * Each "settable" configuration parameter uses the cfg interface, which
 * accepts either strings (for initialisation during startup) or actual
 * parameter values (from, e.g., the HUD).
 */
enum	cfgkey {
	CFGKEY_ALIGN = 0,
	CFGKEY_ALIGNFLOCK,
	CFGKEY_BIRDSPERFLOCK,
	CFGKEY_BPS,
	CFGKEY_COHESION,
	CFGKEY_DIMENSIONS,
	CFGKEY_GROUND,
	CFGKEY_PALIGN,
	CFGKEY_PALIGNFLOCK,
	CFGKEY_PAYOFFS,
	CFGKEY_PLAYINTERVAL,
	CFGKEY_ROOSTS,
	CFGKEY_SCATTERMEAN,
	CFGKEY_SEPARATION,
	CFGKEY_SPEED_MAX,
	CFGKEY_SPEED_MIN,
	CFGKEY_TEND_RDEST,
	CFGKEY_TEND_ROOST,
	CFGKEY_TEND_FROM_ROOST,
	CFGKEY_WINDOW,
	CFGKEY__MAX
};

enum	fecfgkey {
	FECFGKEY_AUTOTRACK,
	FECFGKEY_FPS,
	FECFGKEY_HUDBIG,
	FECFGKEY_HUDLASTPLAY,
	FECFGKEY_HUDMINI,
	FECFGKEY_LIGHTPOS,
	FECFGKEY_MAXFRAMES,
	FECFGKEY_MAXMATCHES,
	FECFGKEY_NOISE0,
	FECFGKEY_OBJECTS,
	FECFGKEY_OUTPUT,
	FECFGKEY_RES,
	FECFGKEY_SCALE,
	FECFGKEY_TEXTURES,
	FECFGKEY_WINDOW,
	FECFGKEY__MAX
};


__BEGIN_DECLS

int	 cfg_scalar_double(struct game *, enum cfgkey, double);
int	 cfg_scalar_int(struct game *, enum cfgkey, int);
int	 cfg_string(struct game *, enum cfgkey, const char *);
int	 cfg_check(struct game *, const char **);
int	 cfg_parsekey(char *, char **, size_t, const char *const *);

int	 mainGL(int, char *[], char *[], char *[]);

int	 objLoadAll(struct args *, const char *);
int	 texLoadAll(struct args *, const char *);

void	 hudSpecial(struct args *, int, int, int);
void	 hudMouse(struct args *, int, int, int, int);
void	 hudDraw(struct args *);

void	 vector_crossproduct(const struct vector *, 
		const struct vector *, struct vector *);
double	 vector_dotproduct(const struct vector *, const struct vector *);
void	 vector_norm(struct vector *);
void	 vector_zero(struct vector *);

extern const char *cfgkeys[CFGKEY__MAX];
extern const char *fecfgkeys[FECFGKEY__MAX];

__END_DECLS

#endif
