/*	$Id: vector.c,v 1.11 2015/06/02 13:03:52 kristaps Exp $ */
/*
 * Copyright (c) 2014 Kristaps Dzonsons <kristaps@kcons.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <float.h>
#include <math.h>
#include <stdint.h>
#include <stdlib.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include <gd.h>
#include <gsl/gsl_rng.h>
#include <flocks.h>
#include <SDL_mixer.h>

#include "extern.h"

void
vector_norm(struct vector *into)
{
	double	 mag;

	mag = sqrt(into->x * into->x +
		    into->y * into->y +
		    into->z * into->z);

	into->x /= mag;
	into->y /= mag;
	into->z /= mag;
}

void
vector_crossproduct(const struct vector *op1, 
	const struct vector *op2, struct vector *into)
{

	into->x = op1->y * op2->z - op1->z * op2->y;
	into->y = op1->z * op2->x - op1->x * op2->z;
	into->z = op1->x * op2->y - op1->y * op2->x;
}

double
vector_dotproduct(const struct vector *op1, const struct vector *op2)
{

	return(op1->x * op2->x + op1->y * op2->y + op1->z * op2->z);
}

void
vector_zero(struct vector *into)
{

	into->x = into->y = into->z = 0.0;
}
